@@ -46,6 +46,8 @@ use tokio::sync;
 use tracing::*;
 use tracing_futures::Instrument;
 
+pub const INIT_MUTEX_TIMEOUT_SECS: u64 = 30;
+
 mod validation_package;
 
 #[allow(missing_docs)]
@@ -97,6 +99,7 @@ where
     cache: EnvWrite,
     holochain_p2p_cell: P2pCell,
     queue_triggers: QueueTriggers,
+    init_mutex: tokio::sync::Mutex<()>,
 }
 
 impl Cell {
@@ -145,6 +148,7 @@ impl Cell {
                     cache,
                     holochain_p2p_cell,
                     queue_triggers,
+                    init_mutex: Default::default(),
                 },
                 initial_queue_triggers,
             ))
@@ -805,6 +809,14 @@ impl Cell {
     /// Check if each Zome's init callback has been run, and if not, run it.
     #[tracing::instrument(skip(self))]
     async fn check_or_run_zome_init(&self) -> CellResult<()> {
+        // Ensure that only one init check is run at a time
+        let _guard = tokio::time::timeout(
+            std::time::Duration::from_secs(INIT_MUTEX_TIMEOUT_SECS),
+            self.init_mutex.lock(),
+        )
+        .await
+        .map_err(|_| CellError::InitTimeout)?;
+
         // If not run it
         let env = self.env.clone();
         let keystore = env.keystore().clone();
@@ -864,7 +876,9 @@ impl Cell {
     //        TaskManager can have these Cell TaskManagers as children.
     //        [ B-04176 ]
     pub async fn cleanup(&self) -> CellResult<()> {
-        tracing::info!("Cell removed, but cleanup is not yet implemented.");
+        use holochain_p2p::HolochainP2pCellT;
+        self.holochain_p2p_cell().leave().await?;
+        tracing::info!("Cell removed, but cleanup is not yet fully implemented.");
         Ok(())
     }
 
@@ -911,3 +925,9 @@ impl Cell {
         &self.queue_triggers
     }
 }
+
+impl std::fmt::Debug for Cell {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Cell").field("id", &self.id()).finish()
+    }
+}
