@@ -11,8 +11,8 @@ pub fn verify_signature(
     call_context: Arc<CallContext>,
     input: VerifySignature,
 ) -> Result<bool, WasmError> {
-    match HostFnAccess::from(&call_context.host_access()) {
-        HostFnAccess { keystore: Permission::Allow, .. } => tokio_helper::block_forever_on(async move {
+    match HostFnAccess::from(&call_context.host_context()) {
+        HostFnAccess { keystore_deterministic: Permission::Allow, .. } => tokio_helper::block_forever_on(async move {
             input
                 .key
                 .verify_signature_raw(input.as_ref(), input.as_data_ref())
@@ -153,7 +153,7 @@ pub mod wasm_test {
                     TestWasm::Sign,
                     "verify_signature_raw",
                     VerifySignature::new_raw(k.clone(), sig.clone().into(), data.clone())
-                );
+                ).unwrap();
 
                 assert_eq!(expect, output_raw, "raw: {}", name);
             }
@@ -179,6 +179,6 @@ pub mod wasm_test {
             TestWasm::Sign,
             "verify_signature",
             fake_agent_pubkey_1()
-        );
+        ).unwrap();
     }
 }
