@@ -6,7 +6,7 @@ use super::guest_callback::validate::ValidateHostAccess;
 use super::guest_callback::validation_package::ValidationPackageHostAccess;
 use super::host_fn::get_agent_activity::get_agent_activity;
 use super::host_fn::HostFnApi;
-use super::HostAccess;
+use super::HostContext;
 use super::ZomeCallHostAccess;
 use crate::core::ribosome::error::RibosomeError;
 use crate::core::ribosome::error::RibosomeResult;
@@ -49,6 +49,9 @@ use crate::core::ribosome::host_fn::get_details::get_details;
 use crate::core::ribosome::host_fn::get_link_details::get_link_details;
 use crate::core::ribosome::host_fn::get_links::get_links;
 use crate::core::ribosome::host_fn::hash_entry::hash_entry;
+use crate::core::ribosome::host_fn::must_get_entry::must_get_entry;
+use crate::core::ribosome::host_fn::must_get_header::must_get_header;
+use crate::core::ribosome::host_fn::must_get_valid_element::must_get_valid_element;
 use crate::core::ribosome::host_fn::query::query;
 use crate::core::ribosome::host_fn::random_bytes::random_bytes;
 use crate::core::ribosome::host_fn::remote_signal::remote_signal;
@@ -173,16 +176,40 @@ impl HostFnBuilder {
 }
 
 type ContextMap = Lazy<Arc<Mutex<HashMap<u64, Arc<CallContext>>>>>;
+/// Map from an instance to it's context for a call.
 static CONTEXT_MAP: ContextMap = Lazy::new(Default::default);
 
-type InstanceCache = HashMap<u64, Arc<Mutex<Instance>>>;
-type ZomeKey = [u8; 32];
-type DnaZomeKey = (DnaHash, ZomeKey);
-type DnaZomeCache = Lazy<Arc<Mutex<HashMap<DnaZomeKey, InstanceCache>>>>;
-static INSTANCE_CACHE: DnaZomeCache = Lazy::new(Default::default);
 static CONTEXT_KEY: AtomicU64 = AtomicU64::new(0);
 
-const INSTANCE_CACHE_SIZE: usize = 20;
+/// Create a key for the instance cache.
+/// It will be [WasmHash..DnaHash..context_key] all as bytes.
+fn instance_cache_key(wasm_hash: &WasmHash, dna_hash: &DnaHash, context_key: u64) -> [u8; 32] {
+    let mut bits = [0u8; 32];
+    for (i, byte) in wasm_hash
+        .get_raw_32()
+        .iter()
+        .zip(dna_hash.get_raw_32().iter())
+        .map(|(a, b)| a ^ b)
+        .take(24)
+        .enumerate()
+    {
+        bits[i] = byte;
+    }
+    for (i, byte) in (24..32).zip(context_key.to_le_bytes()) {
+        bits[i] = byte;
+    }
+    bits
+}
+
+/// Get the context key back from the end of the instance cache key.
+fn context_key_from_key(key: &[u8; 32]) -> u64 {
+    let mut bits = [0u8; 8];
+    for (a, b) in key[24..].iter().zip(bits.iter_mut()) {
+        *b = *a;
+    }
+
+    u64::from_le_bytes(bits)
+}
 
 impl RealRibosome {
     /// Create a new instance
@@ -222,22 +249,25 @@ impl RealRibosome {
         instance: Arc<Mutex<Instance>>,
         zome_name: &ZomeName,
     ) -> RibosomeResult<()> {
+        use holochain_wasmer_host::module::PlruCache;
         // Clear the context as the call is done.
         {
             CONTEXT_MAP.lock().remove(&context_key);
         }
-        let cache_key = (
-            self.dna_file.dna_hash().clone(),
-            self.wasm_cache_key(&zome_name)?,
+        let key = instance_cache_key(
+            &self
+                .dna_file
+                .dna()
+                .get_wasm_zome(zome_name)
+                .map_err(DnaError::from)?
+                .wasm_hash,
+            self.dna_file.dna_hash(),
+            context_key,
         );
-        // Lock the cache.
-        let mut lock = INSTANCE_CACHE.lock();
-        if let Some(cache) = lock.get_mut(&cache_key) {
-            // If we have space in the cache then add this instance.
-            if cache.len() <= INSTANCE_CACHE_SIZE {
-                cache.insert(context_key, instance);
-            }
-        }
+        holochain_wasmer_host::module::INSTANCE_CACHE
+            .write()
+            .put_item(key, instance);
+
         Ok(())
     }
 
@@ -245,6 +275,7 @@ impl RealRibosome {
         &self,
         call_context: CallContext,
     ) -> RibosomeResult<(Arc<Mutex<Instance>>, u64)> {
+        use holochain_wasmer_host::module::PlruCache;
         let zome_name = call_context.zome.zome_name().clone();
 
         // Fallback to creating an instance if we don't have a cache hit.
@@ -256,65 +287,64 @@ impl RealRibosome {
             ));
             RibosomeResult::Ok(instance)
         };
-        {
-            let zome_key = (
-                self.dna_file.dna_hash().clone(),
-                self.wasm_cache_key(&zome_name)?,
-            );
-            // Lock the cache.
-            let mut lock = INSTANCE_CACHE.lock();
-            // Check if we have a hit for this zome.
-            match lock.get_mut(&zome_key) {
-                Some(cache) => {
-                    // We have a hit for the zome.
-                    let key = cache.keys().next().cloned();
-                    // Check if we have a hit for the instance.
-                    if let Some(context_key) = key {
-                        // We have a key, check if we have an instance hit.
-                        if let Some(instance) = cache.remove(&context_key) {
-                            // We have an instance hit.
-                            // Update the context.
-                            {
-                                CONTEXT_MAP
-                                    .lock()
-                                    .insert(context_key, Arc::new(call_context));
-                            }
-                            // This is the fastest path.
-                            return Ok((instance, context_key));
-                        }
-                    }
 
-                    // We didn't get an instance hit so create a new key.
-                    let context_key = CONTEXT_KEY.fetch_add(1, std::sync::atomic::Ordering::SeqCst);
-                    // Update the context.
-                    {
-                        CONTEXT_MAP
-                            .lock()
-                            .insert(context_key, Arc::new(call_context));
-                    }
-                    // Fallback to creating the instance.
-                    let instance = fallback(context_key)?;
-                    Ok((instance, context_key))
-                }
-                None => {
-                    // We didn't get a hit for the zome so create a cache for this zome.
-                    let cache = HashMap::new();
-                    // Get the key for this context.
-                    let context_key = CONTEXT_KEY.fetch_add(1, std::sync::atomic::Ordering::SeqCst);
-                    // Update the context.
-                    {
-                        CONTEXT_MAP
-                            .lock()
-                            .insert(context_key, Arc::new(call_context));
-                    }
-                    // Fallback to creating the instance.
-                    let instance = fallback(context_key)?;
-                    // Insert the cache for next time.
-                    lock.insert(zome_key, cache);
-                    Ok((instance, context_key))
+        // Get the start of the possible keys.
+        let key_start = instance_cache_key(
+            &self
+                .dna_file
+                .dna()
+                .get_wasm_zome(&zome_name)
+                .map_err(DnaError::from)?
+                .wasm_hash,
+            self.dna_file.dna_hash(),
+            0,
+        );
+        // Get the end of the possible keys.
+        let key_end = instance_cache_key(
+            &self
+                .dna_file
+                .dna()
+                .get_wasm_zome(&zome_name)
+                .map_err(DnaError::from)?
+                .wasm_hash,
+            self.dna_file.dna_hash(),
+            CONTEXT_KEY.load(std::sync::atomic::Ordering::Relaxed),
+        );
+        let mut lock = holochain_wasmer_host::module::INSTANCE_CACHE.write();
+        // Get the first available key.
+        let key = lock
+            .cache()
+            .range(key_start..key_end)
+            .next()
+            .map(|(k, _)| k)
+            .cloned();
+        // Check if we got a key hit.
+        if let Some(key) = key {
+            // If we did then remove that instance.
+            if let Some(instance) = lock.remove_item(&key) {
+                let context_key = context_key_from_key(&key);
+                // We have an instance hit.
+                // Update the context.
+                {
+                    CONTEXT_MAP
+                        .lock()
+                        .insert(context_key, Arc::new(call_context));
                 }
+                // This is the fastest path.
+                return Ok((instance, context_key));
             }
         }
+        // We didn't get an instance hit so create a new key.
+        let context_key = CONTEXT_KEY.fetch_add(1, std::sync::atomic::Ordering::SeqCst);
+        // Update the context.
+        {
+            CONTEXT_MAP
+                .lock()
+                .insert(context_key, Arc::new(call_context));
+        }
+        // Fallback to creating the instance.
+        let instance = fallback(context_key)?;
+        Ok((instance, context_key))
     }
 
     fn imports(&self, context_key: u64, store: &Store) -> ImportObject {
@@ -386,6 +416,9 @@ impl RealRibosome {
             .with_host_function(&mut ns, "__get_links", get_links)
             .with_host_function(&mut ns, "__get_link_details", get_link_details)
             .with_host_function(&mut ns, "__get_agent_activity", get_agent_activity)
+            .with_host_function(&mut ns, "__must_get_entry", must_get_entry)
+            .with_host_function(&mut ns, "__must_get_header", must_get_header)
+            .with_host_function(&mut ns, "__must_get_valid_element", must_get_valid_element)
             .with_host_function(&mut ns, "__query", query)
             .with_host_function(&mut ns, "__call_remote", call_remote)
             .with_host_function(&mut ns, "__remote_signal", remote_signal)
@@ -411,10 +444,17 @@ macro_rules! do_callback {
         let mut results: Vec<(ZomeName, $callback_result)> = Vec::new();
         // fallible iterator syntax instead of for loop
         let mut call_iterator = $self.call_iterator($access.into(), $invocation);
-        while let Some(output) = call_iterator.next()? {
-            let (zome, callback_result) = output;
-            let zome_name: ZomeName = zome.into();
-            let callback_result: $callback_result = callback_result.into();
+        loop {
+            let (zome_name, callback_result): (ZomeName, $callback_result) =
+                match call_iterator.next() {
+                    Ok(Some((zome, extern_io))) => (zome.into(), extern_io.decode()?),
+                    Err((zome, RibosomeError::WasmError(wasm_error))) => (
+                        zome.into(),
+                        <$callback_result>::try_from_wasm_error(wasm_error)?,
+                    ),
+                    Err((_zome, other_error)) => return Err(other_error),
+                    Ok(None) => break,
+                };
             // return early if we have a definitive answer, no need to keep invoking callbacks
             // if we know we are done
             if callback_result.is_definitive() {
@@ -436,14 +476,14 @@ impl RibosomeT for RealRibosome {
     /// if it does not exist then return Ok(None)
     fn maybe_call<I: Invocation>(
         &self,
-        host_access: HostAccess,
+        host_context: HostContext,
         invocation: &I,
         zome: &Zome,
         to_call: &FunctionName,
     ) -> Result<Option<ExternIO>, RibosomeError> {
         let call_context = CallContext {
             zome: zome.clone(),
-            host_access,
+            host_context,
         };
 
         match zome.zome_def() {
@@ -486,10 +526,10 @@ impl RibosomeT for RealRibosome {
 
     fn call_iterator<I: crate::core::ribosome::Invocation>(
         &self,
-        access: HostAccess,
+        host_context: HostContext,
         invocation: I,
     ) -> CallIterator<Self, I> {
-        CallIterator::new(access, self.clone(), invocation)
+        CallIterator::new(host_context, self.clone(), invocation)
     }
 
     /// Runs the specified zome fn. Returns the cursor used by HDK,
@@ -505,9 +545,10 @@ impl RibosomeT for RealRibosome {
             let fn_name = invocation.fn_name.clone();
 
             let guest_output: ExternIO =
-                match self.call_iterator(host_access.into(), invocation).next()? {
-                    Some(result) => result.1,
-                    None => return Err(RibosomeError::ZomeFnNotExists(zome_name, fn_name)),
+                match self.call_iterator(host_access.into(), invocation).next() {
+                    Ok(Some((_zome, extern_io))) => extern_io,
+                    Ok(None) => return Err(RibosomeError::ZomeFnNotExists(zome_name, fn_name)),
+                    Err((_zome, ribosome_error)) => return Err(ribosome_error),
                 };
 
             ZomeCallResponse::Ok(guest_output)
@@ -526,7 +567,7 @@ impl RibosomeT for RealRibosome {
         access: GenesisSelfCheckHostAccess,
         invocation: GenesisSelfCheckInvocation,
     ) -> RibosomeResult<GenesisSelfCheckResult> {
-        do_callback!(self, access, invocation, GenesisSelfCheckResult)
+        do_callback!(self, access, invocation, ValidateCallbackResult)
     }
 
     fn run_validate(
@@ -612,12 +653,12 @@ pub mod wasm_test {
         host_access.workspace = workspace;
 
         let foo_result: String =
-            crate::call_test_ribosome!(host_access, TestWasm::HdkExtern, "foo", ());
+            crate::call_test_ribosome!(host_access, TestWasm::HdkExtern, "foo", ()).unwrap();
 
         assert_eq!("foo", foo_result.as_str());
 
         let bar_result: String =
-            crate::call_test_ribosome!(host_access, TestWasm::HdkExtern, "bar", ());
+            crate::call_test_ribosome!(host_access, TestWasm::HdkExtern, "bar", ()).unwrap();
 
         assert_eq!("foobar", bar_result.as_str());
     }
